searchData={"items":[{"type":"module","title":"StrongParams","doc":"It filters request params keeping only explicitly enumerated parameters.","ref":"StrongParams.html"},{"type":"macro","title":"StrongParams.filter_for/2","doc":"Macro to add filter for action parameters.\n\nIt adds a `Plug` to filter request params before `Phoenix` call the respective\ncontroller action. This macro must be called inside a Phoenix controller implementation.\n\nThe first given argument must be a valid action name. The second must be a `Keyword` opts\nwith the list of required and permitted parameters. The `Keyword` may have both lists or\njust one of them. Optionaly you can pass `forbidden_params_err` as true to enforce error\nfor received params not listed as permitted or required.\n\n  * `:permitted` - List of parameters to keep. If some of listed parameters is missing no error is returned.\n  * `:required` - List of parameters that are required. In case of missing parameters a error will be returned with a map enumerating the missing parameters.\n  * `:forbidden_params_err` - Boolean value to enforce error for received params not listed. Default is false.\n\n```elixir\nfilter_for(:create, required: [:name, :email], permitted: [:nickname], forbidden_params_err: true)\nfilter_for(:update, required: [:name, :email], permitted: [:nickname])\n```\n\nFor nested parameters you must use a keyword.\n\nExemple:\n\n```elixir\nfilter_for(:index, required: [:name, :email, address: [:street, :city]], permitted: [:nickname])\n\n# Expected filtered parameters\n%{\n   name: \"Johnny Lawrence\",\n   nickname: \"John\",\n   email: \"john@mail.com\",\n   address: %{\n     street: \"5º Avenue\",\n     city: \"NY\"\n   }\n}\n```\n\nFor a list of params you must use a nested list\n\nExample:\n\n```elixir\nfilter_for(:create, required: [:name, attachments: [[:name]]])\n\n# Expected filtered parameters\n%{\n   name: \"Johnny Lawrence\",\n   attachments: [\n     %{name: \"file.jpg\"},\n     %{name: \"doc.pdf\"}\n   ]\n }\n```","ref":"StrongParams.html#filter_for/2"},{"type":"macro","title":"Cast value - StrongParams.filter_for/2","doc":"`Ecto.Type` is used to the casting, so `ecto` needs to be\navailable as a dependency in your app.\n\nAdd to your `mix.exs`.\n\n```elixir\n{:ecto, \"~> x.x\"}\n```\n\nTo cast values you must provide a tuple {field, type}\n\nExample:\n\n```elixir\nfilter_for(:create, required: [{:id, Ecto.UUID}, {:date, {:array, :date}}])\n\n# Expected filtered parameters\n%{\n   id: \"11268bd3-5e41-4e6f-bf28-f3e167f87767\",\n   dates: [~D[2021-11-29], ~D[2021-11-30]]\n }\n```\n\nAny custom `Ecto.Type` or [ecto primitive types](https://hexdocs.pm/ecto/Ecto.Schema.html#module-primitive-types)\nare valid types.","ref":"StrongParams.html#filter_for/2-cast-value"},{"type":"type","title":"StrongParams.filters/0","doc":"","ref":"StrongParams.html#t:filters/0"},{"type":"type","title":"StrongParams.parameters_list/0","doc":"","ref":"StrongParams.html#t:parameters_list/0"},{"type":"module","title":"StrongParams.Error","doc":"This module defines a `t:StrongParams.Error.t/0` struct that stores filtering errors.\n\nWhen using `Phoenix` action_fallback the fallback module will receive a tuple of `{:error, %StrongParams.Error{}}`\n\n```elixir\ndefmodule YourPhoenixApp.Fallback do\n\n  def call(conn, %StrongParams.Error{}) do\n    send_resp(conn, 400, \"Your custom msg\")\n  end\nend\n```","ref":"StrongParams.Error.html"},{"type":"type","title":"StrongParams.Error.errors_map/0","doc":"","ref":"StrongParams.Error.html#t:errors_map/0"},{"type":"type","title":"StrongParams.Error.t/0","doc":"","ref":"StrongParams.Error.html#t:t/0"},{"type":"extras","title":"Get starting","doc":"# StrongParams\n\n[![GitHub Workflow Status](https://img.shields.io/github/workflow/status/Finbits/strong_params/CI?style=flat-square)](https://github.com/Finbits/strong_params/actions?query=workflow%3ACI)\n[![Hex.pm](https://img.shields.io/hexpm/v/strong_params?style=flat-square)](https://hex.pm/packages/strong_params)\n[![Hex.pm](https://img.shields.io/hexpm/l/strong_params?style=flat-square)](https://hex.pm/packages/strong_params)\n[![Hex.pm](https://img.shields.io/hexpm/dt/strong_params?style=flat-square)](https://hex.pm/packages/strong_params)\n[![codecov](https://img.shields.io/codecov/c/github/Finbits/strong_params?style=flat-square)](https://codecov.io/gh/Finbits/strong_params)\n\nInspired by Ruby on Rails Strong Parameters. It filters request params keeping only explicitly enumerated parameters.\n\nIn addition, parameters can be marked as required and flow through a controller fallback flow to end up as a 400 Bad Request with no effort.","ref":"readme.html"},{"type":"extras","title":"Installation - Get starting","doc":"Add `StrongParams` to your application\n\nmix.exs\n\n```elixir\ndef deps do\n  [\n    {:strong_params, \"~> 0.4.0\"}\n  ]\nend\n```\n\nUpdate deps\n\n```sh\nmix deps.get\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Get starting","doc":"`StrongParams` uses macros to apply the parameters filter. You must `use` _StrongParams_\nin each controller you want filter parameters. The better way is using _StrongParams_\nin you Phoenix App entrypoint inside `controller/0` block (it must be add after\nuse `Phoenix.Controller`).\n\n/lib/your_phoenix_app.ex\n\n```elixir\ndefmodule YourPhoenixApp do\n  ...\n\n  def controller do\n    quote do\n      use Phoenix.Controller, namespace: YourPhoenixApp\n      use StrongParams\n\n      ...\n\n    end\n  end\nend\n```\n\nThen you can use macro `filter_for/2` inside your controller to apply the filters\nfor each action.\n\n```elixir\ndefmodule YourPhoenixApp.UserController do\n  use YourPhoenixApp, :controller\n\n  alias YourPhoenixApp.User\n\n  filter_for(:create, required: [:name, :email], permitted: [:nickname])\n\n  def create(conn, %{name: _, email: _} = params) do\n\n    user = %User{}\n      |> User.changeset(params)\n      |> Repo.insert()\n\n    render(conn, user: user)\n  end\n\nend\n```\n\nIn above example, once `filter_for/2` is defined for action `create` the second\nargument received in action will be the filtered request parameters. The map `params`\nhas atomized keys, once the filter is defined as `atoms` (the lib don't create new atoms).\n\nIf some parameter enumerated as required is missing in received request the `Plug.Conn`\nwill be halted with status code `400` and a generic error message. If you add an\naction fallback you can handle the filter error:\n\n```elixir\ndefmodule YourPhoenixApp.UserController do\n  use YourPhoenixApp, :controller\n\n  action_fallback(YourPhoenixApp.Fallback)\n\n  filter_for(:create, required: [:name, :email], permitted: [:nickname])\n\n  ...\n\nend\n\n\ndefmodule YourPhoenixApp.Fallback do\n  alias StrongParams.Error\n\n  def call(conn, %Error{errors: errors}) do\n    send_resp(conn, 400, \"Your custom msg #{inpect(errors)}\")\n  end\nend\n```\n\nYou must call `filter_for/2` for each action you want to filter the params.\n\n```elixir\nfilter_for(:create, required: [:name, :email], permitted: [:nickname])\nfilter_for(:update, permitted: [:name, :email, :nickname])\n```","ref":"readme.html#usage"},{"type":"extras","title":"Contributing - Get starting","doc":"[Contributing Guide](CONTRIBUTING.md)","ref":"readme.html#contributing"},{"type":"extras","title":"License - Get starting","doc":"[Apache License, Version 2.0](LICENSE) © [Finbits](https://github.com/Finbits)","ref":"readme.html#license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}